;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 1 test for TEB bands. Calculates the SNR and NEdT. Fits the SNR
;  to the BCS delta radinace (using a function [fit type=0] or a quadratic polynomial [fit
;  type=1]) with or without weights of 1/std. RSR is implimented in rsr=0 and not in rsr=1.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) SNR[collect,band,HAM,detector,subsample,0=scan method 1=subsample method 2=overall method]
;  2) snr_1[band,HAM,detector,subsample,coefficients,fit type,not weighted or weighted,rsr]
;  2) snr_1_error[band,HAM,detector,subsample,coefficients,fit type,not weighted or weighted,rsr]
;  3) snr_fit[collect,band,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;  4) NEdL[collect,b,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;  5) NEdT[collect,band,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;  6) SNR_typ[band,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;  7) NEdL_typ[band,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;  8) NEdT_typ[band,HAM,detector,subsample,fit type,not weighted or weighted,rsr]
;
;  v1.0 - 04/03/2008 - Jeff McIntire
;  v2.0 - 03/23/2009 - Jeff McIntire
;   -- Replaced Powell's method for the minimization with curvefit.
;   -- Added error estimates on the fit coefficients.
;*****************************************************************************

;*****************************************************************************
;  Planck derivative
;*****************************************************************************

FUNCTION Planck_deriv, T,lambda

c1=14387.86D				; h*c/k*10^6
c2=119106211.8D				; 2*h*c^2*10^(6*5)

return,(c1*c2/(lambda^6*T^2))*(exp(c1/(lambda*T))/(exp(c1/(lambda*T))-1.)^2)

END

;*****************************************************************************
;  Functions for curvefit.
;*****************************************************************************

; SNR equal to the Delta radiance divided by sqrt of quadratic polynominal in delta radiance.
PRO curfunc1, X,A, F, pder

F=X/sqrt(A[0]+A[1]*X+A[2]*X^2)
FOR i=0, n_elements(X)-1 DO BEGIN
    IF A[0]+A[1]*X[i]+A[2]*X[i]^2 LT 0.0 THEN F[i]=100000
ENDFOR

IF N_PARAMS() GE 4 THEN pder=[[-X/(2*sqrt(A[0]+A[1]*X+A[2]*X^2)^3)],[-X^2/(2*sqrt(A[0]+A[1]*X+A[2]*X^2)^3)], $
                              [-X^3/(2*sqrt(A[0]+A[1]*X+A[2]*X^2)^3)]]

END

; Quadratic polynomial.
PRO curfunc2, X,A, F, pder

F=A[0]+A[1]*X+A[2]*X^2

IF N_PARAMS() GE 4 THEN pder=[[replicate(1.0,n_elements(X))],[X],[X^2]]

END

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_SNR_NEdT, infile

nlines=File_Lines(infile)
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
maxscan=intarr(1)               ; # of scans
sample1=intarr(1)               ; 1st M band sample
nsamples=intarr(1)              ; # of M band samples

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,maxscan
readf,lun,sample1
readf,lun,nsamples
Free_Lun,lun

maxscans=maxscan[0]

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,'general.TEB.2.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.eff_rad.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.snr.sav'

;*****************************************************************************
;  SNR definitions.
;*****************************************************************************

SNR_temp=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3)
SNR=dblarr(ncollects,nbands,nham,maxdet,maxsubsets)
sigma_SNR_temp=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3)
sigma_SNR=dblarr(ncollects,nbands,nham,maxdet,maxsubsets)

;*****************************************************************************
;  Calculate the SNR (scan method).
;*****************************************************************************

SNR_1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans)
SNR_2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets)
SNR_3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans)

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    SNR_1[col,b,ih,id,is,0:maxscans-1]=dn_col[col,b,ih,id,is,1,0:maxscans-1,0]/std_col[col,b,ih,id,is,1,0:maxscans-1,0]
                    FOR iscans=0, maxscans-1 DO BEGIN
                        IF std_col[col,b,ih,id,is,1,iscans,0] EQ 0 THEN SNR_1[col,b,ih,id,is,iscans]=0
                    ENDFOR
                    
                    SNR_2[col,b,ih,id,is]=sig_mean(SNR_1[col,b,ih,id,is,0:maxscans-1],3,1)
                    FOR iscans=0, maxscans-1 DO SNR_3[col,b,ih,id,is,iscans]=(SNR_1[col,b,ih,id,is,iscans]-SNR_2[col,b,ih,id,is])^2
                    
                    SNR_temp[col,b,ih,id,is,0]=sig_mean(SNR_1[col,b,ih,id,is,0:maxscans-1],3,1)
                    sigma_SNR_temp[col,b,ih,id,is,0]=sqrt(sig_mean(SNR_3[col,b,ih,id,is,0:maxscans-1],3,1))
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the SNR (sample method).
;*****************************************************************************

SNR_1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,nsamples[0]*2)
SNR_2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets)
SNR_3=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,nsamples[0]*2)

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        IF band_type[b] EQ 1 THEN max_frame=nsamples[0]*2
        IF band_type[b] EQ 0 THEN max_frame=nsamples[0]
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    SNR_1[col,b,ih,id,is,0:max_frame-1]=dn_col_2[col,b,ih,id,is,1,0:max_frame-1]/std_col_2[col,b,ih,id,is,1,0:max_frame-1]
                    FOR f=0, max_frame-1 DO BEGIN
                        IF std_col_2[col,b,ih,id,is,1,f] EQ 0 THEN SNR_1[col,b,ih,id,is,f]=0
                    ENDFOR
                    
                    SNR_2[col,b,ih,id,is]=sig_mean(SNR_1[col,b,ih,id,is,0:max_frame-1],3,1)
                    FOR f=0, max_frame-1 DO SNR_3[col,b,ih,id,is,f]=(SNR_1[col,b,ih,id,is,f]-SNR_2[col,b,ih,id,is])^2
                    
                    SNR_temp[col,b,ih,id,is,1]=sig_mean(SNR_1[col,b,ih,id,is,0:max_frame-1],3,1)
                    sigma_SNR_temp[col,b,ih,id,is,1]=sqrt(sig_mean(SNR_3[col,b,ih,id,is,0:max_frame-1],3,1))
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the SNR (overall method).
;*****************************************************************************

SNR_temp[*,*,*,*,*,2]=dn[*,*,*,*,*,1,2]/std[*,*,*,*,*,1,2]
sigma_SNR_temp[*,*,*,*,*,2]=std[*,*,*,*,*,1,2]

;*****************************************************************************
;  Use the largest SNR
;*****************************************************************************

FOR col=0, ncollects-1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    z=WHERE(SNR_temp[col,b,ih,id,is,*] EQ max(SNR_temp[col,b,ih,id,is,*]))

                    SNR[col,b,ih,id,is]=SNR_temp[col,b,ih,id,is,z]
                    sigma_SNR[col,b,ih,id,is]=sigma_SNR_temp[col,b,ih,id,is,2]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Powell's method for fitting SNR to BCS radiance (function and quadratic polynomial).
;*****************************************************************************

w=dblarr(ncollects)
snr_1=dblarr(nbands,nham,maxdet,maxsubsets,3,2,2,2)
snr_1_error=dblarr(nbands,nham,maxdet,maxsubsets,3,2,2,2)
snr_fit=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2,2,2)

FOR ifit=0, 1 DO BEGIN
    ; 0 for rsr correction and 1 for no rsr correction.
    FOR rsr=0, 1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR ih=0, nham-1 DO BEGIN
                    FOR id=0, detectors[b]-1 DO BEGIN
                        ; Use only points within the dynamic range.
                        dyr=WHERE(T_bcs[*] GT T_min[b,0]-0.5 AND T_bcs[*] LT T_max[b,0]+0.5)
         
                        ; Guassian weights.
                        IF ifit EQ 0 THEN w[*]=1.0
                        IF ifit EQ 1 THEN w[*]=1/sigma_SNR[*,b,ih,id,is]^2
                        
                       ; Curvefit method for the SNR function in curfunc.
                        P1=[.000001,.000001,.000001]
                        yfit=curvefit(rad_BCS_ret[dyr,b,ih,id,is,1,rsr],SNR[dyr,b,ih,id,is],w[dyr],P1,SIGMA1,/DOUBLE,FUNCTION_NAME='curfunc1')
                        snr_1[b,ih,id,is,0:2,0,ifit,rsr]=P1[0:2]
                        snr_1_error[b,ih,id,is,0:2,0,ifit,rsr]=SIGMA1[0:2]
                        snr_fit[dyr,b,ih,id,is,0,ifit,rsr]=(SNR[dyr,b,ih,id,is]-rad_BCS_ret[dyr,b,ih,id,is,1,rsr] $
                                                            /sqrt(snr_1[b,ih,id,is,0,0,ifit,rsr] $
                                                              +snr_1[b,ih,id,is,1,0,ifit,rsr]*rad_BCS_ret[dyr,b,ih,id,is,1,rsr] $
                                                              +snr_1[b,ih,id,is,2,0,ifit,rsr]*rad_BCS_ret[dyr,b,ih,id,is,1,rsr]^2))

                       ; Curvefit method for the quadratic polynomial in curfunc.
                        P2=[.000001,.000001,.000001]
                        yfit=curvefit(rad_BCS_ret[dyr,b,ih,id,is,1,rsr],SNR[dyr,b,ih,id,is],w[dyr],P2,SIGMA2,/DOUBLE,FUNCTION_NAME='curfunc2')
                        snr_1[b,ih,id,is,0:2,1,ifit,rsr]=P2[0:2]
                        snr_1_error[b,ih,id,is,0:2,1,ifit,rsr]=SIGMA2[0:2]
                        snr_fit[dyr,b,ih,id,is,1,ifit,rsr]=(SNR[dyr,b,ih,id,is]-(snr_1[b,ih,id,is,0,1,ifit,rsr] $
                                                                                   +snr_1[b,ih,id,is,1,1,ifit,rsr]*rad_BCS_ret[dyr,b,ih,id,is,1,rsr] $
                                                                                   +snr_1[b,ih,id,is,2,1,ifit,rsr]*rad_BCS_ret[dyr,b,ih,id,is,1,rsr]^2))

                        ;print,ifit,rsr,b,id,ih,is,P1[0],P1[1],P1[2],P2[0],P2[1],P2[2]
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calulate the NEdL and NEdT.
;*****************************************************************************

NEdL=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2,2,2)
NEdT=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2,2,2)
SNR_typ=dblarr(nbands,nham,maxdet,maxsubsets,2,2,2)
NEdL_typ=dblarr(nbands,nham,maxdet,maxsubsets,2,2,2)
NEdT_typ=dblarr(nbands,nham,maxdet,maxsubsets,2,2,2)

; 0 for rsr correction and 1 for no rsr correction.
FOR rsr=0, 1 DO BEGIN
    FOR ifit1=0, 1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR col=0, ncollects-1 DO BEGIN
                FOR ifit2=0, 1 DO BEGIN
                    NEdL[col,b,*,*,*,ifit2,ifit1,rsr]=rad_BCS_ret[col,b,*,*,*,1,rsr]/SNR[col,b,*,*,*]
                    NEdT[col,b,*,*,*,ifit2,ifit1,rsr]=NEdL[col,b,*,*,*,ifit2,ifit1,rsr]/Planck_deriv(T_bcs[col],lambda_center[b])
                ENDFOR
            ENDFOR
            ; Calculate the SNR, NEdL, and NEdT at Ttyp.
            FOR ih=0, nham-1 DO BEGIN
                FOR is=0, subsets[b]-1 DO BEGIN
                    FOR id=0, detectors[b]-1 DO BEGIN

                        ; From the SNR function.
                        SNR_typ[b,ih,id,is,0,ifit1,rsr]=L_typ[b,0] $
                          /sqrt(snr_1[b,ih,id,is,0,0,ifit1,rsr]+snr_1[b,ih,id,is,1,0,ifit1,rsr]*L_typ[b,0]+snr_1[b,ih,id,is,2,0,ifit1,rsr]*L_typ[b,0]^2)
                        NEdL_typ[b,ih,id,is,0,ifit1,rsr]= $
                          sqrt(snr_1[b,ih,id,is,0,0,ifit1,rsr]+snr_1[b,ih,id,is,1,0,ifit1,rsr]*L_typ[b,0]+snr_1[b,ih,id,is,2,0,ifit1,rsr]*L_typ[b,0]^2)
                        NEdT_typ[b,ih,id,is,0,ifit1,rsr]=NEdL_typ[b,ih,id,is,0,ifit1,rsr]/Planck_deriv(T_typ[b,0],lambda_center[b])

                        ; From the quadratic polynomial.
                        SNR_typ[b,ih,id,is,1,ifit1,rsr]= $
                          (snr_1[b,ih,id,is,0,1,ifit1,rsr]+snr_1[b,ih,id,is,1,1,ifit1,rsr]*L_typ[b,0]+snr_1[b,ih,id,is,2,1,ifit1,rsr]*L_typ[b,0]^2)
                        NEdL_typ[b,ih,id,is,1,ifit1,rsr]=L_typ[b,0] $
                          /(snr_1[b,ih,id,is,0,1,ifit1,rsr]+snr_1[b,ih,id,is,1,1,ifit1,rsr]*L_typ[b,0]+snr_1[b,ih,id,is,2,1,ifit1,rsr]*L_typ[b,0]^2)
                        NEdT_typ[b,ih,id,is,1,ifit1,rsr]=NEdL_typ[b,ih,id,is,1,ifit1,rsr]/Planck_deriv(T_typ[b,0],lambda_center[b])
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
print,'Band    SNR_typ    NEdt_typ    (detector and subsample median)'
FOR b=0, nbands-1 DO BEGIN
;    print,band_name[b],median(SNR_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,0,1,0]),median(NEdT_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,0,1,0]), $
;      median(SNR_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,1,1,0]),median(NEdT_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,1,1,0])
    print,band_name[b],median(SNR_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,0,0,0]),median(NEdT_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,0,0,0]), $
      median(SNR_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,1,0,0]),median(NEdT_typ[b,0,0:detectors[b]-1,0:subsets[b]-1,1,0,0])
ENDFOR

;*****************************************************************************
;  Output SNR and NEdT to txt and sav files.
;*****************************************************************************

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.snr_typ.txt'
outfile3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.NEdT_typ.txt'

fmt_1='(i3, 5x, i2, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f)'

openw,lun,outfile2,/get_lun
printf,lun,'Signal to Noise Ratio (SNR) at T_typ (averaged over subsample, without weighted fits with RSR)'
printf,lun,'HAM      DET     I4        I5       M12       M13        M14      M15      M16A      M16B'
FOR ih=0, nham-1 DO BEGIN
    FOR id=0, maxdet-1 DO BEGIN
        IF id LT 16 THEN printf,lun,format=fmt_1,ih,id,mean(SNR_typ[0,ih,id,0:1,0,0,0]), $
          mean(SNR_typ[1,ih,id,0:1,0,0,0]),SNR_typ[2,ih,id,0,0,0,0],SNR_typ[3,ih,id,0,0,0,0], $
          SNR_typ[4,ih,id,0,0,0,0],SNR_typ[5,ih,id,0,0,0,0],SNR_typ[6,ih,id,0,0,0,0],SNR_typ[7,ih,id,0,0,0,0]
        IF id GE 16 THEN printf,lun,format=fmt_1,ih,id,mean(SNR_typ[0,ih,id,0:1,0,0,0]), $
          mean(SNR_typ[1,ih,id,0:1,0,0,0]),0.0,0.0,0.0,0.0,0.0,0.0
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile3,/get_lun
printf,lun,'Noise Equivalent detla Temperature (NEdT) at T_typ (averaged over subsample, without weighted fits with RSR)'
printf,lun,'HAM      DET     I4        I5       M12       M13        M14      M15      M16A      M16B'
FOR ih=0, nham-1 DO BEGIN
    FOR id=0, maxdet-1 DO BEGIN
        IF id LT 16 THEN printf,lun,format=fmt_1,ih,id,mean(NEdT_typ[0,ih,id,0:1,0,0,0]), $
          mean(NEdT_typ[1,ih,id,0:1,0,0,0]),NEdT_typ[2,ih,id,0,0,0,0],NEdT_typ[3,ih,id,0,0,0,0], $
          NEdT_typ[4,ih,id,0,0,0,0],NEdT_typ[5,ih,id,0,0,0,0],NEdT_typ[6,ih,id,0,0,0,0],NEdT_typ[7,ih,id,0,0,0,0]
        IF id GE 16 THEN printf,lun,format=fmt_1,ih,id,mean(NEdT_typ[0,ih,id,0:1,0,0,0]), $
          mean(NEdT_typ[1,ih,id,0:1,0,0,0]),0.0,0.0,0.0,0.0,0.0,0.0
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary.txt'

fmt_5='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile16,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_5,band_name[b],ih,is,SNR_typ[b,ih,id,is,1,0,0],NEdL_typ[b,ih,id,is,1,0,0],NEdT_typ[b,ih,id,is,1,0,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary1.txt'

fmt_5='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile16,/get_lun
printf,lun,'Band   Collect    Detector   HAM   Subset   T_bcs   L_bcs   SNR(scan)   SNR(sample)   SNR(overall)'
FOR b=0, nbands-1 DO BEGIN
    IF b NE 2 THEN continue

    FOR col=0, ncollects-1 DO BEGIN
        FOR id=0, detectors[b]-1 DO BEGIN
            IF id NE 8 THEN continue

            FOR ih=0, nham-1 DO BEGIN
                FOR is=0, subsets[b]-1 DO BEGIN
                    printf,lun,format=fmt_5,band_name[b],col,id+1,ih,is,T_bcs[col],rad_BCS_ret[col,b,ih,id,is,1,0],SNR_temp[col,b,ih,id,is,0], $
                      SNR_temp[col,b,ih,id,is,1],SNR_temp[col,b,ih,id,is,2]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

save,/compress,SNR,SNR_temp,SNR_typ,snr_1,snr_1_error,snr_fit,NEdL,NEdT,NEdL_typ,NEdT_typ,filename=outfile

END
