;*****************************************************************************
;				NICST TOOL
;
;  Uses polynominal fits, linear (ifit=0),  quadratic (ifit=1) 
;  and cubic (ifit=2), relating dn to radiance. Uses the sample method for dn.
;  Also fit both with RSR (rsr=0) and without (rsr=1).
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) ncollects
;  3) rg_1[band,HAM,detector,subsample,a0 a1 a2 a3,ifit,rsr] -- radiometric coefficients.
;  3) rg_1_error[band,HAM,detector,subsample,a0 a1 a2 a3,ifit,rsr] --
;  coefficient errors.
;  3) gc[,collects,band,HAM,detector,subsample,ifit,rsr,iter] -- gain
;                                                                drift correction.
;  4) dL_fit[collect,band,HAM,detector,subsample,ifit,rsr] -- fractional fitting residual.
;  5) rrcu[band,HAM,detector,subsample,ifit,rsr,sum or sum in quadriture] 
;      -- radiometric response characterization uncertainty.
;  6) rrnl[band,HAM,detector,subsample,ifit,rsr] -- radiometric response non-linearity.
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;  v2.0 - 05/27/2008 - Jeff McIntire
;  v3.0 - 03/20/2009 - Jeff McIntire
;     -- Added gain drift correction and errors on radiometric coefficients.
;*****************************************************************************

PRO RC5_fit, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,'general.TEB.2.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'

;*****************************************************************************
;  Definitions.
;*****************************************************************************

x=dblarr(ncollects)
z=dblarr(ncollects)

rg_1=dblarr(nbands,nham,maxdet,maxsubsets,4,6,2)
rg_1_error=dblarr(nbands,nham,maxdet,maxsubsets,4,6,2)
gc=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2,20)

dL_fit=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,6,2)
mean_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,6,2,2)
abs_max_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,6,2)
stddev_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,6,2,2)
rrcu=dblarr(nbands,nham,maxdet,maxsubsets,6,2,2,2)
rrnl=dblarr(nbands,nham,maxdet,maxsubsets,6,2)

rad_OBC_x=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2)

mindet=[0,0,0,0,1,0,0,0]

;*****************************************************************************
;  Average over scans for delta OBC radiance.
;*****************************************************************************

FOR rsr=0, 1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR ih=0, nham-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    FOR col=0, ncollects-1 DO BEGIN
                        rad_OBC_x[col,b,ih,id,is,rsr]=sig_mean(rad_OBC[col,b,ih,id,is,*,rsr],3,1)                  
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Fitting using POLY_FIT (Linear,Quadratic, and Cubic).
;  ifit=0 for linear, 1 for quadratic, and 2 for cubic POLY_FIT L vs dn (calibration equation).
;  ifit=3 for linear, 4 for quadratic, and 5 for cubic POLY_FIT dn vs L (response equation).
;*****************************************************************************

FOR ifit=0, 5 DO BEGIN
    ;  0 for rsr correction and 1 for no rsr correction.
    FOR rsr=0, 1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR ih=0, nham-1 DO BEGIN
                    FOR id=0, detectors[b]-1 DO BEGIN
                        FOR iter=0, 19 DO BEGIN

                            ; Detemine collect were BCS and OBC temperatures are closest.
                            T_close=min(abs(T_bcs[*]-T_obc[ih,*]))
                            close_col=WHERE(abs(T_bcs[*]-T_obc[ih,*]) EQ T_close)

                            ; Use only points within the dynamic range (minus those with low SNR (below SNR=3). Cut 190 K from M14.
                         ;   dyr=WHERE(T_bcs[*] GT T_min[b,0]-0.5 AND T_bcs[*] LT T_max[b,0]+0.5)
                            dyr=WHERE(T_bcs[*] GT T_min[b,0]-0.5 AND T_bcs[*] LT T_max[b,0]+0.5 AND dn[*,b,ih,id,0,1,0]/std[*,b,ih,id,0,1,0] GT 3.)
                            IF b EQ 4 THEN dyr=WHERE(T_bcs[*] GT T_min[b,0]+0.5 AND T_bcs[*] LT T_max[b,0]+0.5 AND dn[*,b,ih,id,0,1,0]/std[*,b,ih,id,0,1,0] GT 5.)

                            ; Implement the gain drift correction, start with gc=1.
                            IF iter EQ 0 THEN gc[dyr,b,ih,id,is,rsr]=1.
                            IF iter GE 1 THEN BEGIN
                                FOR col=0, ncollects-1 DO BEGIN

                                    ; Get subsample for I bands for OBC dn.
                                    IF dn[col,b,ih,id,0,1,0] GT dn[col,b,ih,id,1,1,0] AND dn_obc[col,b,ih,id,0,1,0] GT dn_obc[col,b,ih,id,1,1,0] THEN BEGIN 
                                        isubset=is
                                    ENDIF ELSE BEGIN
                                        IF is EQ 0 THEN isubset=1
                                        IF is EQ 1 THEN isubset=0
                                    ENDELSE
                                    IF b GE 2 THEN isubset=0

                                    IF ifit EQ 0 OR ifit EQ 3 THEN  gc[col,b,ih,id,is,rsr,iter]= $
                                      (rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2])/rad_OBC_x[col,b,ih,id,is,rsr] $
                                      *rad_OBC_x[close_col,b,ih,id,is,rsr] $
                                      /(rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2])
                                    IF ifit EQ 1 OR ifit EQ 4 THEN  gc[col,b,ih,id,is,rsr,iter]= $
                                      (rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2] $
                                       +rg_1[b,ih,id,is,2,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2]^2)/rad_OBC_x[col,b,ih,id,is,rsr] $
                                      *rad_OBC_x[close_col,b,ih,id,is,rsr] $
                                      /(rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2] $
                                        +rg_1[b,ih,id,is,2,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2]^2)
                                    IF ifit EQ 2 OR ifit EQ 5 THEN  gc[col,b,ih,id,is,rsr,iter]= $
                                      (rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2] $
                                       +rg_1[b,ih,id,is,2,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2]^2 $
                                       +rg_1[b,ih,id,is,3,ifit,rsr]*dn_obc[col,b,ih,id,isubset,1,2]^3)/rad_OBC_x[col,b,ih,id,is,rsr] $
                                      *rad_OBC_x[close_col,b,ih,id,is,rsr] $
                                      /(rg_1[b,ih,id,is,0,ifit,rsr]+rg_1[b,ih,id,is,1,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2] $
                                        +rg_1[b,ih,id,is,2,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2]^2 $
                                        +rg_1[b,ih,id,is,3,ifit,rsr]*dn_obc[close_col,b,ih,id,isubset,1,2]^3)
                                ENDFOR
                            ENDIF
                            
                            ; Set the x and y for the fitting.
                            x=dn[dyr,b,ih,id,is,1,0]
                            z=rad_BCS[dyr,b,ih,id,is,rsr]*gc[dyr,b,ih,id,is,rsr,iter]

                            ; Calbration equation.
                            IF ifit EQ 0 THEN result1=POLY_FIT(x,z,1,SIGMA=error)
                            IF ifit EQ 1 THEN result2=POLY_FIT(x,z,2,SIGMA=error)
                            IF ifit EQ 2 THEN result3=POLY_FIT(x,z,3,SIGMA=error)
                            
                            ; Response equation.
                            IF ifit EQ 3 THEN result1=POLY_FIT(z,x,1,SIGMA=error)
                            IF ifit EQ 4 THEN result2=POLY_FIT(z,x,2,SIGMA=error)
                            IF ifit EQ 5 THEN result3=POLY_FIT(z,x,3,SIGMA=error)

                            ; Radiometric gains.
                            IF ifit EQ 0 OR ifit EQ 3 THEN rg_1[b,ih,id,is,0:1,ifit,rsr]=result1[0:1]
                            IF ifit EQ 1 OR ifit EQ 4 THEN rg_1[b,ih,id,is,0:2,ifit,rsr]=result2[0:2]
                            IF ifit EQ 2 OR ifit EQ 5 THEN rg_1[b,ih,id,is,*,ifit,rsr]=result3[*]
                        
                            ; Errors.
                            IF ifit EQ 0 OR ifit EQ 3 THEN rg_1_error[b,ih,id,is,0:1,ifit,rsr]=error[0:1]
                            IF ifit EQ 1 OR ifit EQ 4 THEN rg_1_error[b,ih,id,is,0:2,ifit,rsr]=error[0:2]
                            IF ifit EQ 2 OR ifit EQ 5 THEN rg_1_error[b,ih,id,is,*,ifit,rsr]=error[*]

                            ; Break loop once gc has converged to within 1*10-5.
                            IF iter NE 0 THEN BEGIN
                                IF abs(mean(gc[dyr,b,ih,id,is,rsr,iter])-mean(gc[dyr,b,ih,id,is,rsr,iter-1])) LT 0.00001 THEN goto,out
                            ENDIF
                        ENDFOR
                        out:
                            
;*****************************************************************************
;  Get the fractional fitting residual for the calibration equation
;  and the fitting residual for the response equation.
;*****************************************************************************

                        IF ifit EQ 0 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]-result1[0]-result1[1]*dn[dyr,b,ih,id,is,1,0])/(rad_BCS[dyr,b,ih,id,is,rsr])
                        IF ifit EQ 1 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]-result2[0]-result2[1]*dn[dyr,b,ih,id,is,1,0] $
                           -result2[2]*dn[dyr,b,ih,id,is,1,0]^2)/(rad_BCS[dyr,b,ih,id,is,rsr])
                        IF ifit EQ 2 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (rad_BCS[dyr,b,ih,id,is,rsr]-result3[0]-result3[1]*dn[dyr,b,ih,id,is,1,0]-result3[2]*dn[dyr,b,ih,id,is,1,0]^2 $
                           -result3[3]*dn[dyr,b,ih,id,is,1,0]^3)/(rad_BCS[dyr,b,ih,id,is,rsr])
                        IF ifit EQ 3 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result1[0]-result1[1]*rad_BCS[dyr,b,ih,id,is,rsr])
                        IF ifit EQ 4 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result2[0]-result2[1]*rad_BCS[dyr,b,ih,id,is,rsr]-result2[2]*(rad_BCS[dyr,b,ih,id,is,rsr])^2)
                        IF ifit EQ 5 THEN dL_fit[dyr,b,ih,id,is,ifit,rsr]= $
                          (dn[dyr,b,ih,id,is,1,0]-result3[0]-result3[1]*rad_BCS[dyr,b,ih,id,is,rsr]-result3[2]*(rad_BCS[dyr,b,ih,id,is,rsr])^2 $
                           -result3[3]*(rad_BCS[dyr,b,ih,id,is,rsr])^3)

;*****************************************************************************
;  Get the radiometric response characterization uncertainty (RRCU),
;  and the radiometric response non-linearity (RRNL).
;*****************************************************************************

                        IF ifit LT 3 THEN BEGIN
                            dyr_1=WHERE(T_bcs[*] GT T_min[b,0]-0.5 AND T_bcs[*] LT 265.)
                            dyr_2=WHERE(T_bcs[*] GT 265. AND T_bcs[*] LT T_max[b,0]+0.5)

                            mean_dL_fit[b,ih,id,is,ifit,rsr,0]=mean(dL_fit[dyr_1,b,ih,id,is,ifit,rsr])
                            stddev_dL_fit[b,ih,id,is,ifit,rsr,0]=stddev(dL_fit[dyr_1,b,ih,id,is,ifit,rsr])
                            
                            mean_dL_fit[b,ih,id,is,ifit,rsr,1]=mean(dL_fit[dyr_2,b,ih,id,is,ifit,rsr])
                            stddev_dL_fit[b,ih,id,is,ifit,rsr,1]=stddev(dL_fit[dyr_2,b,ih,id,is,ifit,rsr])

                            rrcu[b,ih,id,is,ifit,rsr,0,0]=mean_dL_fit[b,ih,id,is,ifit,rsr,0]+stddev_dL_fit[b,ih,id,is,ifit,rsr,0]
                            rrcu[b,ih,id,is,ifit,rsr,0,1]=sqrt(mean_dL_fit[b,ih,id,is,ifit,rsr,0]^2+stddev_dL_fit[b,ih,id,is,ifit,rsr,0]^2)

                            rrcu[b,ih,id,is,ifit,rsr,1,0]=mean_dL_fit[b,ih,id,is,ifit,rsr,1]+stddev_dL_fit[b,ih,id,is,ifit,rsr,1]
                            rrcu[b,ih,id,is,ifit,rsr,1,1]=sqrt(mean_dL_fit[b,ih,id,is,ifit,rsr,1]^2+stddev_dL_fit[b,ih,id,is,ifit,rsr,1]^2)
                        ENDIF
                        IF ifit GE 3 THEN BEGIN
                            abs_max_dL_fit[b,ih,id,is,ifit,rsr]=max(abs(dL_fit[dyr,b,ih,id,is,ifit,rsr]))
                            rrnl[b,ih,id,is,ifit,rsr]=abs_max_dL_fit[b,ih,id,is,ifit,rsr]/(L_max[b,0]*rg_1[b,ih,id,is,1,ifit,rsr])
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
            IF ifit EQ 1 THEN print,'GAIN ',iter,ifit,rsr,STRING(9B),band_name[b],STRING(9B), $
              1/mean(rg_1[b,0,mindet[b]:detectors[b]-1,0:subsets[b]-1,1,ifit,rsr]), $
              1/mean(rg_1[b,1,mindet[b]:detectors[b]-1,0:subsets[b]-1,1,ifit,rsr]), $
              mean(rg_1_error[b,0,mindet[b]:detectors[b]-1,0:subsets[b]-1,1,ifit,rsr]), $
              mean(rg_1_error[b,1,mindet[b]:detectors[b]-1,0:subsets[b]-1,1,ifit,rsr])
            IF ifit EQ 1 THEN print,'RRCU ',iter,ifit,rsr,STRING(9B),band_name[b],STRING(9B), $
              mean(rrcu[b,0,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr,0,1]), $
              mean(rrcu[b,1,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr,0,1]), $
              mean(rrcu[b,0,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr,1,1]), $
              mean(rrcu[b,1,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr,1,1])
            IF ifit EQ 3 THEN print,'RRNL ',iter,ifit,rsr,STRING(9B),band_name[b],STRING(9B), $
              mean(rrnl[b,0,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr]), $
              mean(rrnl[b,1,mindet[b]:detectors[b]-1,0:subsets[b]-1,ifit,rsr])
        ENDFOR
    ENDFOR
ENDFOR

save,/compress,ncollects,rg_1,rg_1_error,gc,dL_fit,rrcu,rrnl,filename=outfile

;*****************************************************************************
;  Ouput gains, rrcu, and rrnl to txt files.
;*****************************************************************************

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.txt'
outfile3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.avg.txt'
outfile4=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.max.txt'
outfile5=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.avg.txt'
outfile6=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.med.txt'
outfile7=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.max.txt'
outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.avg.txt'
outfile9=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.med.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
fmt_2='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
fmt_3='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'
fmt_4='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'
fmt_5='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
fmt_6='(a5, 5x, i2, 5x, i2, 5x, d)'

openw,lun,outfile2,/get_lun
printf,lun,'Radiametric gains and errors (with RSR)'
printf,lun,'Band  Detector  HAM  SUBSET   Gain(linear)   Gain(quad)   Gain(cubic)   Error(linear)   Error(quad)   Error(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name[b],idet+1,ih,is,rg_1[b,ih,idet,is,1,3,0],rg_1[b,ih,idet,is,1,4,0],rg_1[b,ih,idet,is,1,5,0], $
                  rg_1_error[b,ih,idet,is,1,3,0],rg_1_error[b,ih,idet,is,1,4,0],rg_1_error[b,ih,idet,is,1,5,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile3,/get_lun
printf,lun,'Radiametric gains and errors (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   Gain(linear)   Gain(quad)   Gain(cubic)   Error(linear)   Error(quad)   Error(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is, $
              mean(rg_1[b,ih,0:detectors[b]-1,is,1,3,0]),mean(rg_1[b,ih,0:detectors[b]-1,is,1,4,0]),mean(rg_1[b,ih,0:detectors[b]-1,is,1,5,0]), $
              mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,3,0]),mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,4,0]),mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,5,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile4,/get_lun
printf,lun,'RRCU (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET   Lmin - 265 [RRCU(linear)  RRCU(quad)  RRCU(cubic)]  265 - Lmax [RRCU(linear)  RRCU(quad)  RRCU(cubic)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_5,band_name[b],ih,is, $
              max(rrcu[b,ih,0:detectors[b]-1,is,0,0,0,1]),max(rrcu[b,ih,0:detectors[b]-1,is,1,0,0,1]),max(rrcu[b,ih,0:detectors[b]-1,is,2,0,0,1]), $
              max(rrcu[b,ih,0:detectors[b]-1,is,0,0,1,1]),max(rrcu[b,ih,0:detectors[b]-1,is,1,0,1,1]),max(rrcu[b,ih,0:detectors[b]-1,is,2,0,1,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
		
openw,lun,outfile5,/get_lun
printf,lun,'RRCU (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   Lmin - 265 [RRCU(linear)  RRCU(quad)  RRCU(cubic)]  265 - Lmax [RRCU(linear)  RRCU(quad)  RRCU(cubic)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_5,band_name[b],ih,is, $
              mean(rrcu[b,ih,0:detectors[b]-1,is,0,0,0,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,1,0,0,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,2,0,0,1]), $
              mean(rrcu[b,ih,0:detectors[b]-1,is,0,0,1,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,1,0,1,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,2,0,1,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile6,/get_lun
printf,lun,'RRCU (detector median with RSR)'
printf,lun,'Band  HAM  SUBSET   Lmin - 265 [RRCU(linear)  RRCU(quad)  RRCU(cubic)]  265 - Lmax [RRCU(linear)  RRCU(quad)  RRCU(cubic)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_5,band_name[b],ih,is, $
              median(rrcu[b,ih,0:detectors[b]-1,is,0,0,0,1]),median(rrcu[b,ih,0:detectors[b]-1,is,1,0,0,1]),median(rrcu[b,ih,0:detectors[b]-1,is,2,0,0,1]), $
              median(rrcu[b,ih,0:detectors[b]-1,is,0,0,1,1]),median(rrcu[b,ih,0:detectors[b]-1,is,1,0,1,1]),median(rrcu[b,ih,0:detectors[b]-1,is,2,0,1,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile7,/get_lun
printf,lun,'RRNL (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_6,band_name[b],ih,is,max(rrnl[b,ih,0:detectors[b]-1,is,3,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile8,/get_lun
printf,lun,'RRNL (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_6,band_name[b],ih,is,mean(rrnl[b,ih,0:detectors[b]-1,is,3,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile9,/get_lun
printf,lun,'RRNL (detector median with RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_6,band_name[b],ih,is,median(rrnl[b,ih,0:detectors[b]-1,is,3,0])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.no_rsr.txt'
outfile9=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.avg.no_rsr.txt'
outfile10=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.max.no_rsr.txt'
outfile11=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.avg.no_rsr.txt'
outfile12=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.max.no_rsr.txt'
outfile13=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.avg.no_rsr.txt'

openw,lun,outfile8,/get_lun
printf,lun,'Radiametric gains (no RSR)'
printf,lun,'Band  Detector  HAM  SUBSET   Gain(linear)   Gain(quad)  Gain(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name[b],idet+1,ih,is,rg_1[b,ih,idet,0,1,3,1],rg_1[b,ih,idet,0,1,4,1],rg_1[b,ih,idet,is,1,5,1], $
                  rg_1_error[b,ih,idet,is,1,3,1],rg_1_error[b,ih,idet,is,1,4,1],rg_1_error[b,ih,idet,is,1,5,1]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile9,/get_lun
printf,lun,'Radiametric gains (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET   Gain(linear)   Gain(quad)  Gain(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is, $
              mean(rg_1[b,ih,0:detectors[b]-1,is,1,3,1]),mean(rg_1[b,ih,0:detectors[b]-1,is,1,4,1]),mean(rg_1[b,ih,0:detectors[b]-1,is,1,5,1]), $
              mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,3,1]),mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,4,1]),mean(rg_1_error[b,ih,0:detectors[b]-1,is,1,5,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile10,/get_lun
printf,lun,'RRCU (worst case detector no RSR)'
printf,lun,'Band  HAM  SUBSET   Lmin - 265 [RRCU(linear)  RRCU(quad)  RRCU(cubic)]  265 - Lmax [RRCU(linear)  RRCU(quad)  RRCU(cubic)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_5,band_name[b],ih,is, $
              max(rrcu[b,ih,0:detectors[b]-1,is,0,1,0,1]),max(rrcu[b,ih,0:detectors[b]-1,is,1,1,0,1]),max(rrcu[b,ih,0:detectors[b]-1,is,2,1,0,1]), $
              max(rrcu[b,ih,0:detectors[b]-1,is,0,1,1,1]),max(rrcu[b,ih,0:detectors[b]-1,is,1,1,1,1]),max(rrcu[b,ih,0:detectors[b]-1,is,2,1,1,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
		
openw,lun,outfile11,/get_lun
printf,lun,'RRCU (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET   Lmin - 265 [RRCU(linear)  RRCU(quad)  RRCU(cubic)]  265 - Lmax [RRCU(linear)  RRCU(quad)  RRCU(cubic)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_5,band_name[b],ih,is, $
              mean(rrcu[b,ih,0:detectors[b]-1,is,0,1,0,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,1,1,0,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,2,1,0,1]), $
              mean(rrcu[b,ih,0:detectors[b]-1,is,0,1,1,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,1,1,1,1]),mean(rrcu[b,ih,0:detectors[b]-1,is,2,1,1,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile12,/get_lun
printf,lun,'RRNL (worst case detector no RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_4,band_name[b],ih,is, $
              max(rrnl[b,ih,0:detectors[b]-1,is,0,1]),max(rrnl[b,ih,0:detectors[b]-1,is,1,1]),max(rrnl[b,ih,0:detectors[b]-1,is,2,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile13,/get_lun
printf,lun,'RRNL (detector averaged no RSR)'
printf,lun,'Band  HAM  SUBSET  RRNL(linear)  RRNL(quad)  RRNL(cubic)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_4,band_name[b],ih,is, $
              mean(rrnl[b,ih,0:detectors[b]-1,is,0,1]),mean(rrnl[b,ih,0:detectors[b]-1,is,1,1]),mean(rrnl[b,ih,0:detectors[b]-1,is,2,1])
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Ouput residual, coefficients, and dn@Lmax to txt files.
;*****************************************************************************

outfile14=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.txt'
outfile15=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.txt'

fmt_3='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
fmt_4='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile14,/get_lun
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_BCS   quadratic(RESIDUAL(FRACTIONAL)   RESIDUAL)   cubic(RESIDUAL(FRACTIONAL)   RESIDUAL)'
FOR b=0, nbands-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    IF T_bcs[col] LT T_min[b,0]-0.5 THEN continue
                    IF T_bcs[col] GT T_max[b,0]+0.5 THEN continue
                    
                    printf,lun,format=fmt_3,band_name[b],col,ih,is,id+1,dn[col,b,ih,id,is,1,0],rad_BCS[col,b,ih,id,is,0],dL_fit[col,b,ih,id,is,1,0], $
                      dL_fit[col,b,ih,id,is,1,0]*rad_BCS[col,b,ih,id,is,0],dL_fit[col,b,ih,id,is,2,0],dL_fit[col,b,ih,id,is,2,0]*rad_BCS[col,b,ih,id,is,0]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile15,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   quadratic(a0   a1   a2)   error(a0   a1   a2)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_4,band_name[b],ih,is,id+1,rg_1[b,ih,id,is,0,1,0],rg_1[b,ih,id,is,1,1,0],rg_1[b,ih,id,is,2,1,0], $
                  rg_1_error[b,ih,id,is,0,1,0],rg_1_error[b,ih,id,is,1,1,0],rg_1_error[b,ih,id,is,2,1,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.Lmax.txt'

fmt_5='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d)'

openw,lun,outfile16,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   dn@Lmax(response eq)   dn@Lmax(calibration eq)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_5,band_name[b],ih,is,id+1,rg_1[b,ih,id,is,0,4,0]+rg_1[b,ih,id,is,1,4,0]*rad_dyn[b,ih,id,is,2,0,0] $
                  +rg_1[b,ih,id,is,2,4,0]*rad_dyn[b,ih,id,is,2,0,0]^2,1/(1/(2*(rg_1[b,ih,id,is,0,1,0]-rad_dyn[b,ih,id,is,2,0,0])) $
                                                                           *(-rg_1[b,ih,id,is,1,1,0]-sqrt(rg_1[b,ih,id,is,1,1,0]^2-4*rg_1[b,ih,id,is,2,1,0] $
                                                                                                          *(rg_1[b,ih,id,is,0,1,0]-rad_dyn[b,ih,id,is,2,0,0]))))
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary2.txt'

openw,lun,outfile16,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   RRNL'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_5,band_name[b],ih,is,id+1,rrnl[b,ih,id,is,3,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

outfile16=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary4.txt'

openw,lun,outfile16,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   Lmin - 265 [RRCU(quad)]  265 - Lmax [RRCU(quad)]'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_5,band_name[b],ih,is,id+1,rrcu[b,ih,id,is,1,0,0,1],rrcu[b,ih,id,is,1,0,1,1]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

END

